********************************************************************************
*
*       Filename: IMTH08.ASM
*       Revision: 1.00
*       Date: February 24, 1993
*
*       Written By: Mark Johnson
*                   Motorola CSIC Applications
*
*       Assembled Under: P&E Microcomputer Systems IASM08 (Beta Version)
*
*                       ********************************
*                       *       Revision History       *
*                       ********************************
*
*       Revision 1.00   2/24/93 Original Source
********************************************************************************
*
*
*       Program Description:
*
*       This program contains six* integer math routines for the 68HC08 family
*       of microcontrollers.
*
*	*Note: 1)  The 32 x 16 Unsigned divide algorithm was based on     
*		     the one written for the 6805 by Don Weiss and was      
*		     modified to return a 32-bit quotient.
*		 2)  The Table lookup and interpolation algorithm was 	     
*		     based on the one written by Kevin Kilbane and was       
*		     modified to interpolate both positive and negative     
*		     slope linear functions.		
*			
*
********************************************************************************
*
*     Start of main routine
*
*
            ORG     $50          ;RAM address space
*
INTACC1     RMB     4            ;32-bit integer accumulator #1
INTACC2     RMB     4            ;32-bit integer accumulator #2
SPVAL       RMB     2            ;storage for stack pointer value
*
*
            ORG     $6E00        ;ROM/EPROM address space
START       LDHX    #$450        ;load H:X with upper RAM boundary + 1
            TXS                  ;move stack pointer to upper RAM boundary
            CLRH                 ;clear H:X
********************************************************************************
*     Unsigned 16x16 multiply
*
*     This routine multiplies the 16-bit unsigned number stored in
*     locations INTACC1:INTACC1+1 by the 16-bit unsigned number stored in
*     locations INTACC2:INTACC2+1 and places the 32-bit result in locations
*     INTACC1....INTACC1+3 (INTACC1 = MSB.....INTACC1+3 = LSB).
*
********************************************************************************
UMULT16     EQU     *
            PSHA                        ;save acc
            PSHX                        ;save x-reg
            PSHH                        ;save h-reg
            AIS     #-6                 ;reserve six bytes of temporary
                                        ;storage on stack
            CLR     6,SP                ;zero storage for multiplication carry
*
*     Multiply (INTACC1:INTACC1+1) by INTACC2+1
*
            LDX     INTACC1+1           ;load x-reg w/multiplier lsb
            LDA     INTACC2+1           ;load acc w/multiplicand lsb
            MUL                         ;multiply
            STX     6,SP                ;save carry from multiply
            STA     INTACC1+3           ;store lsb of final result
            LDX     INTACC1             ;load x-reg w/multiplier msb
            LDA     INTACC2+1           ;load acc w/multiplicand lsb
            MUL                         ;multiply
            ADD     6,SP                ;add carry from previous multiply
            STA     2,SP                ;store 2nd byte of interm. result 1.
            BCC     NOINCA              ;check for carry from addition
            INCX                        ;increment msb of interm. result 1.
NOINCA      STX     1,SP                ;store msb of interm. result 1.
            CLR     6,SP                ;clear storage for carry
*
*     Multiply (INTACC1:INTACC1+1) by INTACC2
*
            LDX     INTACC1+1           ;load x-reg w/multiplier lsb
            LDA     INTACC2             ;load acc w/multiplicand msb
            MUL                         ;multiply
            STX     6,SP                ;save carry from multiply
            STA     5,SP                ;store lsb of interm. result 2.
            LDX     INTACC1             ;load x-reg w/multiplier msb
            LDA     INTACC2             ;load acc w/multiplicand msb
            MUL                         ;multiply
            ADD     6,SP                ;add carry from previous multiply
            STA     4,SP                ;store 2nd byte of interm. result 2.
            BCC     NOINCB              ;check for carry from addition
            INCX                        ;increment msb of interm. result 2.
NOINCB      STX     3,SP                ;store msb of interm. result 2.
*
*     Add the intermediate results and store the remaining three bytes of the
*     final value in locations INTACC1....INTACC1+2.
*
            LDA     2,SP                ;load acc with 2nd byte of 1st result
            ADD     5,SP                ;add acc with lsb of 2nd result
            STA     INTACC1+2           ;store 2nd byte of final result
            LDA     1,SP                ;load acc with msb of 1st result
            ADC     4,SP                ;add w/ carry 2nd byte of 2nd result
            STA     INTACC1+1           ;store 3rd byte of final result
            LDA     3,SP                ;load acc with msb from 2nd result
            ADC     #0                  ;add any carry from previous addition
            STA     INTACC1             ;store msb of final result
*
*     Reset stack pointer and recover original register values
*
            AIS     #6                  ;deallocate the six bytes of local
                                        ;storage
            PULH                        ;restore h-reg
            PULX                        ;restore x-reg
            PULA                        ;restore accumulator
            RTS                         ;return
********************************************************************************
********************************************************************************
*
*     Unsigned 32 x 32 Multiply
*
*     This routine multiplies the unsigned 32-bit number stored in locations
*     INTACC1.....INTACC1+3 by the unsigned 32-bit number stored in locations
*     INTACC2.....INTACC2+3 and places the unsigned 64-bit result in locations
*     INTACC1.....INTACC2+3 (INTACCC1 = MSB ..... INTACC2+3 = LSB).
*
********************************************************************************
UMULT32     EQU     *
            PSHA                        ;save acc
            PSHX                        ;save x-reg
            PSHH                        ;save h-reg
            CLRX                        ;zero x-reg
            CLRA                        ;zero accumulator
            AIS     #-35T               ;reserve 35 bytes of temporary storage
                                        ;on stack
            TSX                         ;transfer stack pointer + 1 to H:X
            AIX     #32T                ;add number of bytes in storage table
            STHX    SPVAL               ;save end of storage table value
            AIX     #-32T               ;reset H:X to stack pointer value
*
*     Clear 32 bytes of storage needed to hold the intermediate results
*
INIT        CLR     ,X                  ;xero a byte of storage
            INCX                        ;point to next location
            CPHX    SPVAL               ;check for end of table
            BNE     INIT                ;
*
*     Initialize multiplicand and multiplier byte position pointers,
*     temporary storage for carry from the multiplication process, and
*     intermediate storage location pointer
*
            STA     35T,SP              ;zero storage for multiplication carry
            LDA     #3                  ;load acc w/ 1st byte position
            STA     33T,SP              ;pointer for multiplicand byte
            STA     34T,SP              ;pointer for multiplier byte
            TSX                         ;transfer stack pointer + 1 to H:X
            AIX     #7                  ;position of 1st column in storage
            STHX    SPVAL               ;pointer to interm. storage position
            CLRH                        ;clear h-reg
*
*     Multiply each byte of the multiplicand by each byte of the multiplier
*     and store the intermediate results
*
MULTLP      LDX     33T,SP              ;load x-reg w/multiplicand byte pointer
            LDA     INTACC2,X           ;load acc with multiplicand
            LDX     34T,SP              ;load x-reg w/ multiplier byte pointer
            LDX     INTACC1,X           ;load x-reg w/ multiplier
            MUL                         ;multiply
            ADD     35T,SP              ;add carry from previous multiply
            BCC     NOINC32             ;check for carry from addition
            INCX                        ;increment result msb
NOINC32     STX     35T,SP              ;move result msb to carry
            LDHX    SPVAL               ;load x-reg w/ storage position pointer
            STA     ,X                  ;store intermediate value
            AIX     #-1                 ;decrement storage pointer
            STHX    SPVAL               ;store new pointer value
            CLRH                        ;clear h-reg
            DEC     34T,SP              ;decrement multiplier pointer
            BPL     MULTLP              ;multiply all four bytes of multiplier
                                        ;by one byte of the multiplicand
            LDHX    SPVAL               ;load x-reg w/ storage position pointer
            LDA     35T,SP              ;load acc w/ carry (msb from last mult)
            STA     ,X                  ;store msb of intermediate result
            AIX     #!11                ;add offset for next intermediate
                                        ;result starting position
            STHX    SPVAL               ;store new value
            CLRH                        ;clear h-reg
            CLR     35T,SP              ;clear carry storage
            LDX     #3                  ;
            STX     34T,SP              ;reset multiplier pointer
            DEC     33T,SP              ;point to next multiplicand
            BPL     MULTLP              ;loop until each multiplicand has been
                                        ;multiplied by each multiplier
*
*     Initialize temporary stack variables used in the addition process
*


            TSX                         ;transfer stack pointer to H:X
            AIX     #7                  ;add offset for lsb of result
            STHX    SPVAL               ;store position of lsb
            CLR     35T,SP              ;clear addition carry storage
            LDA     #7                  ;
            STA     33T,SP              ;store lsb position of final result
            LDA     #3                  ;
            STA     34T,SP              ;store counter for number of rows
*
*     add all four of the enties in each column together and store the
*     final 64-bit value in locations INTACC1.....INTACC2+3.
*
OUTADDLP    LDA     35T,SP              ;load acc with carry
            CLR     35T,SP              ;clear carry
INADDLP     ADD     ,X                  ;add entry in table to accumulator
            BCC     ADDFIN              ;check for carry
            INC     35T,SP              ;increment carry
ADDFIN      AIX     #8                  ;load H:X with position of next entry
                                        ;column
            DEC     34T,SP              ;decrement row counter
            BPL     INADDLP             ;loop until all four entries in column
                                        ;have been added together
            CLRH                        ;clear h-reg
            LDX     #3                  ;
            STX     34T,SP              ;reset row pointer
            LDX     33T,SP              ;load final result byte pointer
            STA     INTACC1,X           ;store one byte of final result
            LDHX    SPVAL               ;load original column pointer
            AIX     #-1                 ;decrement column pointer
            STHX    SPVAL               ;store new pointer value
            DEC     33T,SP              ;decrement final result byte pointer
            BPL     OUTADDLP            ;loop until all eight columns have
                                        ;been added up and the final results
                                        ;stored
*
*     Reset stack pointer and recover original registers values
*
            AIS     #35T                ;deallocate local storage
            PULH                        ;restore h-reg
            PULX                        ;restore x-reg
            PULA                        ;restore accumulator
            RTS                         ;return
********************************************************************************
********************************************************************************
*
*     Signed 8 x 8 Multiply
*
*     This routine multiplies the signed 8-bit number stored in location
*     INTACC1 by the signed 8-bit number stored in location INTACC2
*     and places the signed 16-bit result in INTACC1:INTACC1+1.
*
*
SMULT8      EQU     *
            PSHX                        ;save x-reg
            PSHA                        ;save accumulator
            PSHH                        ;save h-reg
            AIS     #-1                 ;reserve 2 bytes of temp. storage
            CLR     1,SP                ;clear storage for result sign
            BRCLR   7,INTACC1,TEST2     ;check multiplier sign bit
            NEG     INTACC1             ;two's comp number if negative
            INC     1,SP                ;set sign bit for negative number
TEST2       BRCLR   7,INTACC2,SMULT     ;check multiplicand sign bit
            NEG     INTACC2             ;two's comp number if negative
            INC     1,SP                ;set or clear sign bit
SMULT       LDX     INTACC1             ;load x-reg with multiplier
            LDA     INTACC2             ;load acc with multiplicand
            MUL                         ;multiply
            STA     INTACC1+1           ;store result lsb
            STX     INTACC1             ;store result msb
            LDA     1,SP                ;load sign bit
            CMP     #1                  ;check for negative
            BNE     RETURN              ;branch to finish if result is positive
            NEG     INTACC1+1           ;two's comp result lsb
            BCC     NOSUB               ;check for borrow from zero
            NEG     INTACC1             ;two's comp result msb
            DEC     INTACC1             ;decrement result msb for borrow
            BRA     RETURN              ;finished
NOSUB       NEG     INTACC1             ;two's comp result msb without decrement
RETURN      AIS     #1                  ;deallocate temp storage
            PULH                        ;restore h-reg
            PULA                        ;restore accumulator
            PULX                        ;restore x-reg
            RTS                         ;return
********************************************************************************
********************************************************************************
*
*     Signed 16 x 16 multiply
*
*     This routine multiplies the signed 16-bit number in INTACC1:INTACC1+1 by
*     the signed 16-bit number in INTACC2:INTACC2+1 and places the signed 32-bit
*     value in locations INTACC1....INTACC1+3 (INTACC1 = MSB...INTACC1+3 = LSB).
*
*
SMULT16     EQU     *
            PSHX                        ;save x-reg
            PSHA                        ;save accumulator
            PSHH                        ;save h-reg
            AIS     #-1                 ;reserve 1 byte of temp. storage
            CLR     1,SP                ;clear storage for result sign
            BRCLR   7,INTACC1,TST2      ;check multiplier sign bit and negate
                                        ;(two's complement) if set
            NEG     INTACC1+1           ;two's comp multiplier lsb
            BCC     NOSUB1              ;check for borrow from zero
            NEG     INTACC1             ;two's comp multiplier msb
            DEC     INTACC1             ;decrement msb for borrow
            BRA     MPRSIGN             ;finished
NOSUB1      NEG     INTACC1             ;two's comp multiplier msb (no borrow)
MPRSIGN     INC     1,SP                ;set sign bit for negative number
TST2        BRCLR   7,INTACC2,MLTSUB    ;check multiplicand sign bit and negate
                                        ;(two's complement) if set
            NEG     INTACC2+1           ;two's comp multiplicand lsb
            BCC     NOSUB2              ;check for borrow from zero
            NEG     INTACC2             ;two's comp multiplicand msb
            DEC     INTACC2             ;decrement msb for borrow
            BRA     MPCSIGN             ;finished
NOSUB2      NEG     INTACC2             ;two's comp multiplicand msb (no borrow)
MPCSIGN     INC     1,SP                ;set or clear sign bit
MLTSUB      JSR     UMULT16             ;multiply INTACC1 by INTACC2
            LDA     1,SP                ;load sign bit
            CMP     #1                  ;check for negative
            BNE     DONE                ;exit if answer is positive,
                                        ;otherwise two's complement result
            LDX     #3                  ;
COMP        COM     INTACC1,X           ;complement a byte of the result
            DECX                        ;point to next byte to be complemented
            BPL     COMP                ;loop until all four bytes of result
                                        ;have been complemented
            LDA     INTACC1+3           ;get result lsb
            ADD     #1                  ;add a "1" for two's comp
            STA     INTACC1+3           ;store new value
            LDX     #2                  ;
TWSCMP      LDA     INTACC1,X           ;add any carry from the previous
            ADC     #0                  ; addition to the next three bytes
            STA     INTACC1,X           ; of the result and store the new
            DECX                        ; values
            BPL     TWSCMP              ;
DONE        AIS     #1                  ;deallocate temp storage on stack
            PULH                        ;restore h-reg
            PULA                        ;restore accumulator
            PULX                        ;restore x-reg
            RTS                         ;return
********************************************************************************
********************************************************************************
*
*     32 x 16 Unsigned Divide
*
*     This routine takes the 32-bit dividend stored in INTACC1.....INTACC1+3
*     and divides it by the 16-bit divisor stored in INTACC2:INTACC2+1.
*     The quotient replaces the dividend and the remainder replaces the divisor.
*
UDVD32    EQU     *
*
DIVIDEND  EQU     INTACC1+2
DIVISOR   EQU     INTACC2
QUOTIENT  EQU     INTACC1
REMAINDER EQU     INTACC1
*
        PSHH                            ;save h-reg value
        PSHA                            ;save accumulator
        PSHX                            ;save x-reg value
        AIS     #-3                     ;reserve three bytes of temp storage
        LDA     #!32                    ;
        STA     3,SP                    ;loop counter for number of shifts
        LDA     DIVISOR                 ;get divisor msb
        STA     1,SP                    ;put divisor msb in working storage
        LDA     DIVISOR+1               ;get divisor lsb
        STA     2,SP                    ;put divisor lsb in working storage
*
*     Shift all four bytes of dividend 16 bits to the right and clear
*     both bytes of the temporary remainder location
*
        MOV     DIVIDEND+1,DIVIDEND+3   ;shift dividend lsb
        MOV     DIVIDEND,DIVIDEND+2     ;shift 2nd byte of dividend
        MOV     DIVIDEND-1,DIVIDEND+1   ;shift 3rd byte of dividend
        MOV     DIVIDEND-2,DIVIDEND     ;shift dividend msb
        CLR     REMAINDER               ;zero remainder msb
        CLR     REMAINDER+1             ;zero remainder lsb
*
*     Shift each byte of dividend and remainder one bit to the left
*
SHFTLP  LDA     REMAINDER               ;get remainder msb
        ROLA                            ;shift remainder msb into carry
        ROL     DIVIDEND+3              ;shift dividend lsb
        ROL     DIVIDEND+2              ;shift 2nd byte of dividend
        ROL     DIVIDEND+1              ;shift 3rd byte of dividend
        ROL     DIVIDEND                ;shift dividend msb
        ROL     REMAINDER+1             ;shift remainder lsb
        ROL     REMAINDER               ;shift remainder msb
*
*     Subtract both bytes of the divisor from the remainder
*
        LDA     REMAINDER+1             ;get remainder lsb
        SUB     2,SP                    ;subtract divisor lsb from remainder lsb
        STA     REMAINDER+1             ;store new remainder lsb
        LDA     REMAINDER               ;get remainder msb
        SBC     1,SP                    ;subtract divisor msb from remainder msb
        STA     REMAINDER               ;store new remainder msb
        LDA     DIVIDEND+3              ;get low byte of dividend/quotient
        SBC     #0                      ;dividend low bit holds subtract carry
        STA     DIVIDEND+3              ;store low byte of dividend/quotient
*
*     Check dividend/quotient lsb. If clear, set lsb of quotient to indicate
*     successful subraction, else add both bytes of divisor back to remainder
*
        BRCLR   0,DIVIDEND+3,SETLSB     ;check for a carry from subtraction
                                        ;and add divisor to remainder if set
        LDA     REMAINDER+1             ;get remainder lsb
        ADD     2,SP                    ;add divisor lsb to remainder lsb
        STA     REMAINDER+1             ;store remainder lsb
        LDA     REMAINDER               ;get remainder msb
        ADC     1,SP                    ;add divisor msb to remainder msb
        STA     REMAINDER               ;store remainder msb
        LDA     DIVIDEND+3              ;get low byte of dividend
        ADC     #0                      ;add carry to low bit of dividend
        STA     DIVIDEND+3              ;store low byte of dividend
        BRA     DECRMT                  ;do next shift and subtract

SETLSB  BSET    0,DIVIDEND+3            ;set lsb of quotient to indicate
                                        ;successive subtraction
DECRMT  DBNZ    3,SP,SHFTLP             ;decrement loop counter and do next
                                        ;shift
*
*     Move 32-bit dividend into INTACC1.....INTACC1+3 and put 16-bit
*     remainder in INTACC2:INTACC2+1
*
        LDA     REMAINDER               ;get remainder msb
        STA     1,SP                    ;temporarily store remainder msb
        LDA     REMAINDER+1             ;get remainder lsb
        STA     2,SP                    ;temporarily store remainder lsb
        MOV     DIVIDEND,QUOTIENT       ;
        MOV     DIVIDEND+1,QUOTIENT+1   ;shift all four bytes of quotient
        MOV     DIVIDEND+2,QUOTIENT+2   ; 16 bits to the left
        MOV     DIVIDEND+3,QUOTIENT+3   ;
        LDA     1,SP                    ;get final remainder msb
        STA     INTACC2                 ;store final remainder msb
        LDA     2,SP                    ;get final remainder lsb
        STA     INTACC2+1               ;store final remainder lsb
*
*     Deallocate local storage, restore register values, and return from
*     subroutine
*
        AIS     #3                      ;deallocate temporary storage
        PULX                            ;restore x-reg value
        PULA                            ;restore accumulator value
        PULH                            ;restore h-reg value
        RTS                             ;return
********************************************************************************
********************************************************************************
*
*     Table Lookup and Interpolation
*
*     This subroutine performs table lookup and interpolation between two 16-bit
*     dependent variables (Y) from a table of up to 256 enties (512 bytes) and
*     allowing up to 256 interpolation levels between entries. INTACC1 contains
*     the position of ENTRY2 and INTACC1+1 contains the interpolation fraction.
*     The 16-bit result is placed in INTACC1+2=msb, INTACC1+3=lsb. INTACC2 is
*     used to hold the two 16-bit entries during the routine.
*
*     Y = ENTRY1 + (INTPFRC(ENTRY2 - ENTRY1))/256
*
TBLINT   EQU       *
*
ENTNUM   EQU       INTACC1              ;position of entry2 (0-255)
INTPFRC  EQU       INTACC1+1            ;interpolation fraction (1-255)/256
RESULT   EQU       INTACC1+2            ;16-bit interpolated Y value
ENTRY1   EQU       INTACC2              ;16-bit enrty from table
ENTRY2   EQU       INTACC2+2            ;16-bit entry from table
*
         PSHH                           ;save h-register
         PSHA                           ;save accumulator
         PSHX                           ;save x-reg
         AIS       #-1                  ;allocate one byte of temp storage
         CLRH                           ;zero h-reg
         CLRA                           ;zero accumulator
         CLR       1,SP                 ;clear storage for difference sign
*
*     Load H:X with position of ENTRY2
*
         LDX       ENTNUM               ;get position of entry2 (0-255)
         LSLX                           ;multiply by 2 (for 16-bit entries)
         BCC       GETENT               ;if overflow from multiply occured,
                                        ;increment H-reg.
         INCA                           ;accumulator = 1
         PSHA                           ;push accumulator value on stack
         PULH                           ;transfer acc. value to h register
*
*     Get both entries from table, subtract ENTRY1 from ENTRY2 and store the
*     16-bit result.
*
GETENT   LDA       TABLE-2,x            ;get entry1 lsb
         STA       ENTRY1
         LDA       TABLE-1,x            ;get entry1 msb
         STA       ENTRY1+1
         LDA       TABLE,x              ;get entry2 msb
         STA       ENTRY2
         LDA       TABLE+1,x            ;get entry2 lsb
         STA       ENTRY2+1
         SUB       ENTRY1+1             ;entry2(lsb) - entry1(lsb)
         STA       RESULT+1             ;store result lsb
         LDA       ENTRY2
         SBC       ENTRY1               ;entry2(msb) - entry1(msb)
         STA       RESULT               ;store result msb
*
*
*     Two's complement 16-bit result if ENTRY1 was greater than ENTRY2, else
*     go do multiply
*
*
         TSTA                           ;test result msb for negative
         BGE       MLTFRAC              ;go do multiply if postive
         INC       1,SP                 ;set sign flag for negative result
         NEG       RESULT+1             ;two's complement result lsb
         BCC       NODECR               ;check for borrow from zero
         NEG       RESULT               ;two's complement result msb
         DEC       RESULT               ;decrement result msb for borrow
         BRA       MLTFRAC              ;go do multiply
NODECR   NEG       RESULT               ;two's comp result msb (no borrow)
*
*     (INTPFRC(RESULT:RESULT+1))/256 = Interpolated result
*
*     Multiply result by interpolation fraction
*
MLTFRAC  LDA       INTPFRC              ;get interpolation fraction
         LDX       RESULT+1             ;get result lsb
         MUL                            ;multiply
         STX       RESULT+1             ;store upper 8-bits of result and throw
                                        ;away lower 8-bits (divide by 256)
         LDA       INTPFRC              ;get interpolation fraction
         LDX       RESULT               ;get result msb
         MUL                            ;multiply
         ADD       RESULT+1             ;add result lsb to lower 8-bits of
                                        ;product
         STA       RESULT+1             ;store new result lsb
         TXA                            ;get upper 8-bits of product
         ADC       #0                   ;add carry from last addition
         STA       RESULT               ;store result msb
*
*     Y = ENTRY1 + Interpolated result
*
*     Check sign flag to determine if interpolated result is to be added to
*     or subtracted from ENTRY1
*
         TST       1,SP                 ;test sign flag for negative
         BLE       ADDVAL               ;if not set, add interpolated result
                                        ;to entry1, else subtract
         LDA       ENTRY1+1             ;get entry1 lsb
         SUB       RESULT+1             ;subtract result lsb
         STA       RESULT+1             ;store new result lsb
         LDA       ENTRY1               ;get entry1 msb
         SBC       RESULT               ;subtact w/ carry result msb
         STA       RESULT               ;store new result msb
         BRA       TBLDONE              ;finished
ADDVAL   LDA       RESULT+1             ;get result lsb
         ADD       ENTRY1+1             ;add entry1 lsb
         STA       RESULT+1             ;store new result lsb
         LDA       ENTRY1               ;get entry1 msb
         ADC       RESULT               ;add w/ carry result msb
         STA       RESULT               ;store new result msb
*
*     Deallocate local storage, restore register values, and return from
*     subroutine.
*
TBLDONE  AIS       #1                   ;deallocate local storage
         PULX                           ;restore x-reg
         PULA                           ;restore accumulator
         PULH                           ;restore h-reg
         RTS                            ;return from subroutine
*
*     Sample of 16-bit table entries
*
TABLE    EQU       *
         FDB       !0000                ;entry 0
         FDB       !32767               ;entry 1
         FDB       !2416                ;entry 2
         FDB       !4271                ;entry 3
********************************************************************************

